#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include "wmgeneral.h"

#include "wmpalm-master.xpm"
#include "wmpalm-mask.xbm"

#define IMAGE_STANDARD	0
#define	IMAGE_HIGHLIGHT	1
#define SYNCCMD 	 "pilot-xfer -s ~/.palm"
#define DEVPILOTPATH "/dev/pilot"
/* this is one hundreth of the time you want it to sleep */
#define SLEEP100		 2500L
#define CONFFILE	 "/.wmpalm"
#define VERSION	"0.1b"

/* global variables */
int devpilot;

char synccmd[256+1] = SYNCCMD;
char devpilotpath[128+1] = DEVPILOTPATH;
char conffile[128+1];

/* function forwards */
int initapp(int argc, char *argv[]);
void loop();
void setimage(int numimage);
void dosync();
int verbose = 0;

int main(int argc, char *argv[])
{
	int retcode,i;
	FILE *cfile;
	char buffer[128+1];
	char *index;
	
	/* preset conffile */
	if (!getenv("HOME"))
	{
		fprintf(stderr,"could not retrieve HOME directory\n");
		exit(1);
	}
	sprintf(conffile,"%s%s",getenv("HOME"),CONFFILE);	

	/* here we should parse the command line */
	for (i = 1; i < argc; i++)
	{
		if ( (strcmp(argv[i],"-v") == 0) || (strcmp(argv[i],"--verbose") == 0) )
		{
			verbose = 1;
		}	
		else if ( (strcmp(argv[i],"-c") == 0) || (strcmp(argv[i],"-f") == 0) || (strcmp(argv[i],"--conffile") == 0) )
		{
			if (i == argc-1)
			{
				fprintf(stderr, "No file given\n");
				exit(1);
			}
			else
			{
				strncpy(conffile,argv[i+1],128);
				if (verbose)
					printf("conffile is now %s\n", conffile);
			}
		}
		else if ( (strcmp(argv[1],"--help") == 0) || (strcmp(argv[1],"-h") == 0) || (strcmp(argv[1],"-?") == 0) )
		{
			printf(	"\nwmpalm version %s\nby Robert Lemmen <robertle@cube.net>\n\n" \
					"Usage:\t wmpalm [-h|--help] [-v|--verbose] [(-c|-f|--conffile) config_file]\n" \
					"\t -display should also work\n\n" \
					"the following assignments are possible in the conffile:\n" \
					"\tDevPilot somedevice\n\tSyncCmd somecommand\n\n", VERSION);
			exit(1);
		}
	}

	/* read the conf file */
	if ( (cfile = fopen(conffile,"r")) )
	{
		while (fgets(buffer, 128, cfile))
		{
			index = buffer;
			while (*index == ' ')
				index++;
			if (strncmp(index,"DevPilot",strlen("DevPilot")) == 0)
			{
				index += strlen("DevPilot");
				while (*index == ' ')
					index++;
				strncpy(devpilotpath, index, 128);
				index = strchr(devpilotpath,'\n');
				if (index)
					*index = '\0';
				if (verbose)
					printf("devpilotpath is now %s\n", devpilotpath);
			}
			else if (strncmp(index,"SyncCmd",strlen("SyncCmd")) == 0)
			{
				index += strlen("SyncCmd");
				while (*index == ' ')
					index++;
				strncpy(synccmd, index, 256);
				index = strchr(synccmd,'\n');
				if (index)
					*index = '\0';
				if (verbose)
					printf("synccmd is now %s\n", synccmd);
			}
			
		}
		fclose(cfile);
		if (verbose)
			printf("conffile read\n");
		
	}
	else
	{
		fprintf(stderr,"could not open file \"%s\" because of error 0x%08X\n", conffile, errno);
		exit(1);
	}
	
	/* now init the stuff */
	if ( (retcode = initapp(argc, argv)) )
	{
		fprintf(stderr, "could not init the application\n");
		exit(retcode);
	}	
	
	/* and enter our loop */
	loop();	
	if (verbose)
		printf("returned from loop\n");
	
	return 0;

}
/* this does the sync */
void dosync()
{
	setimage(IMAGE_HIGHLIGHT);
	RedrawWindow();
	if (verbose)
		printf("the command returned %i\n",system(synccmd));
	setimage(IMAGE_STANDARD);
	RedrawWindow();
	return;
}

/* this sets a specific image */
void setimage(int numimage)
{
	copyXPMArea((numimage+1)*64,0,64,64,0,0);
}

/* this checks for events in a loop */
void loop()
{
	XEvent Event;
    fd_set descset;
    struct timeval tv;
	int retval;
	static long actsleep = SLEEP100;

	while (1)
	{
		while (XPending(display)) 
		{
			XNextEvent(display, &Event);
			actsleep = SLEEP100;
			switch (Event.type) 
			{
				case Expose:
					if (verbose)
						printf("Event: Expose\n");
					RedrawWindow();
					break;
				case DestroyNotify:
					if (verbose)
						printf("Event: DestroyNotify\n");
					XCloseDisplay(display);
					exit(0);
					break;
				case ButtonRelease:
					if (verbose)
						printf("Event: Buttonpressed...");
					switch (Event.xbutton.button) 
					{
						case 1:
							if (verbose)
								printf("left\n");
							dosync();
							break;
						case 2:
							if (verbose)
								printf("middle\n");
							break;
						case 3:
							if (verbose)
								printf("right\n");
							break;
					}
			}
		}	
		/* this is a dirty trick to raise a sigpipe if the window has been destroyed, we simple write to it */
		copyXPMArea(1,0,1,1,0,0);
		/* now check wether something is moving on /dev/pilot */
		FD_ZERO(&descset);
		FD_SET(devpilot, &descset);
        tv.tv_sec = 0;
        tv.tv_usec = actsleep;

        retval = select(devpilot+1,&descset,NULL,NULL,&tv);
		
		if (retval)
		{
			if (verbose)
				printf("doing the sync...\n");
			dosync();
		}
		else
		{
			if (actsleep < (SLEEP100*100))
			{
				actsleep += SLEEP100;
				if (verbose)
					printf("actsleep is now %li\n", actsleep);
			}	
		}	
		
	}
	return;
}

/* this build the window, returns non-zero on failure */
int initapp(int argc, char *argv[])
{
	devpilot = open(devpilotpath,O_RDWR);
	openXwindow(argc, argv, wmpalm_master_xpm, wmpalm_mask_bits, wmpalm_mask_width, wmpalm_mask_height);
	setimage(IMAGE_STANDARD);
	AddMouseRegion(0, 0, 0, wmpalm_mask_width, wmpalm_mask_height);
	
	return 0;
}
